<?php
use Softneta\MedDream\Core\Constants;
use Softneta\MedDream\Core\Backend;
use Softneta\MedDream\Core\Audit;
use Softneta\MedDream\Core\Logging;
use Softneta\MedDream\Core\Configuration;
use Softneta\MedDream\Core\InstallValidator;

$error = '';
$study = null;
$viewer = null;
$token = null;

try
{
	ini_set('session.cookie_httponly', 1);		/* for the PHPSESSID cookie */
	session_start();
	require_once('autoload.php');

	global $log, $authDB, $audit;	/* used in external.php */

	if (isset($_REQUEST['study']))
		$study = htmlspecialchars($_REQUEST['study']);
	if (isset($_REQUEST['viewer']))
		$viewer = $_REQUEST['viewer'];
	if (isset($_REQUEST['token']))
		$token = htmlspecialchars($_REQUEST['token']);

	$log = new Logging();
	$backend = new Backend(array('Auth'), true, $log);
	$authDB = $backend->authDB;
	$external = @file_exists("external.php");

	$authDB->addXFrameOptionsHdr();

	if (is_null($study) || is_null($viewer) || is_null($token))
		$error = 'Missing parameter(s)';
	else
	{
		if ($external)
		{
			$audit = new Audit('EXTERNAL');
			require_once('external.php');

			/* hide possibly malicious input from $viewer */
			if (($viewer != 'html') && ($viewer != 'flash'))
				$viewer = 'html';

			/* make sure a correct external.sample-*.php was used */
			if (!function_exists('validateToken'))
				$error = "Missing function 'validateToken' in external.php";

			if (!$error)
			{
				/* validateToken() in external.php reacts to the value; this time default behavior is needed */
				unset($_SESSION['validated_token']);

				$r = check_study_uid($study, $msg, $unused);
				if ($r)
					/* disable validateToken() in external.php for the next time

						If there is no frontend window, then a new one will be opened via traditional
						HIS integration where token validation is mandatory. But, single-use tokens
						are no less important. Therefore validateToken() will just remove this variable
						instead of checking the token, and subsequent calls will have the validation
						enabled.

						In case we are adding a study to an existing frontend window, then the backend
						isn't involved and this variable will stay a lot longer. However that is not a
						big problem.
					 */
					$_SESSION['validated_token'] = $token;
				else
					$error = "UID validation failed ($msg)";
			}
		}
		else
			$error = 'Missing external.php';
	}

	/* anti-XSS: do not include potential rubbish etc in the HTML output */
	if (strlen($error))
		$study = $viewer = $token = '';

/* display our exception message (most are configuration-related) */
}
catch (Exception $e)
{
	$error = $e->getMessage();
	echo "<html><body>\n";
	echo "$error<br>\n";
	if (Constants::EXCEPTIONS_NO_EXIT > 1)
	{
		echo "\nException trace:\n<pre>";
		echo $e->getTraceAsString() . "</pre>\n";
	}
	echo "</body></html>\n";
}
?>
<!DOCTYPE html>
<html>
	<head>
		<script src="swf/AC_OETags.js" type="text/javascript"></script>
		<script>
			function getUrl() {
				var currentURL = window.location.href;
				currentURL = currentURL.replace("addStudy.php", "");
				currentURL = currentURL.replace("index.php", "");
				currentURL = currentURL.replace(location.search, '');
				return currentURL;
			}

			function addStudyToMedDream(uuid, view, token) {
				var error = "<?php echo $error ?>";
				if (error) {
					alert(error);
					return;
				}

				// fall back to HTML if Flash won't work
				if (view != "html") {
					var hasRequestedFlashVersion = DetectFlashVer(9, 0, 0);
					if (!hasRequestedFlashVersion)
						view = "html";
				}

				// fall back to Flash if IE web browser detected
				if ((detectIeVersion() !== false && detectIeVersion() <= 11)
						|| (typeof(Storage) !== `undefined` && localStorage.getItem('enabledOnlyFlashViewer') == 'true')) {
					view = "flash";
				}

				if (view != "html") {
					var currentURL = getUrl() + '?token=' + token + "&htmlMode=off&study=" + uuid;
						// keep "study=..." at the end: win.addStudy() is too dumb, see swf/index.php
					var win = window.open("", "MedDreamFlashViewer");
					if (typeof win.addStudy !== "undefined") {
						win.addStudy(currentURL);
					} else {
						win.document.location = currentURL;
					}
					window.close();
				} else {
					var currentURL = getUrl()+ "?study=" + uuid + '&token=' + token + "&htmlMode=on";
					var win = window.open("", "MedDreamViewer");
					if (win.gui && typeof win.gui.loadStudy !== "undefined") {
						win.gui.loadStudy(uuid);
						// win.gui.loadStudy(uuid, 'study') === win.gui.loadStudy(uuid);
						// win.gui.loadStudy(uuid, 'series');
						// win.gui.loadStudy(uuid, 'image');
					} else {
						win.document.location = currentURL;
					}
					window.close();
				}
			}

			function detectIeVersion() {
				var ua = window.navigator.userAgent;

				var msie = ua.indexOf('MSIE ');
				if (msie > 0) {
					return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
				}

				var trident = ua.indexOf('Trident/');
				if (trident > 0) {
					var rv = ua.indexOf('rv:');
					return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
				}

				var edge = ua.indexOf('Edge/');
				if (edge > 0) {
					return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
				}

				// other browser
				return false;
			}

		</script>
	</head>
	<body onload='addStudyToMedDream("<?php echo $study; ?>", "<?php echo $viewer; ?>", "<?php echo $token; ?>")'>
	</body>
</html>
